
// Some stuff copied from the Broadcom examples, ofcoz!

#include <math.h>
#include <stdlib.h>
#include <SDL.h>

#ifdef RASPI
 #include <bcm_host.h>
 #include <GLES/gl.h>
 #include <EGL/egl.h>
 #include <EGL/eglext.h>
#else
 #include <SDL_opengl.h>
#endif

#include "koelli.h"

#ifdef RASPI
static EGLDisplay display;
static EGLSurface surface;
static EGLContext context;
#endif

static uint32_t width,height,buf_width,buf_height,flagga,depthi;

void koelli_open(int xres,int yres,int depth,int flags)
{
#ifdef RASPI
    static const EGLint attribute_list[] =
    {
        EGL_RED_SIZE, 8,
        EGL_GREEN_SIZE, 8,
        EGL_BLUE_SIZE, 8,
        EGL_ALPHA_SIZE, 8,
        EGL_DEPTH_SIZE, 16,
        EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
        EGL_NONE
    };

    EGLint num_config;
    EGLConfig config;

    static EGL_DISPMANX_WINDOW_T nativewindow;

    // Sheesh. Could it be a bit MORE complicated?
    DISPMANX_ELEMENT_HANDLE_T dispman_element;
    DISPMANX_DISPLAY_HANDLE_T dispman_display;
    DISPMANX_UPDATE_HANDLE_T dispman_update;
    VC_RECT_T dst_rect;
    VC_RECT_T src_rect;
#endif

    int hx,hy;

    buf_width=xres;
    buf_height=yres;
    flagga=flags;
    depthi=depth;

    const SDL_VideoInfo *inho;
    SDL_Init(SDL_INIT_VIDEO);
    inho=SDL_GetVideoInfo();

#ifdef RASPI
    // This is sort of useless, only needed for the kbd
    SDL_SetVideoMode(inho->current_w,inho->current_h,16,SDL_HWSURFACE);

    bcm_host_init();

    display=eglGetDisplay(EGL_DEFAULT_DISPLAY);
    eglInitialize(display,NULL,NULL);
    eglChooseConfig(display,attribute_list,&config,1,&num_config);
    context=eglCreateContext(display,config,EGL_NO_CONTEXT,NULL);

    graphics_get_display_size(0,&width,&height);

    dst_rect.x=0;
    dst_rect.y=0;
    dst_rect.width=width;
    dst_rect.height=height;

    src_rect.x=0;
    src_rect.y=0;
    src_rect.width=width<<16;
    src_rect.height=height<<16;

    dispman_display=vc_dispmanx_display_open(0);
    dispman_update=vc_dispmanx_update_start(0);

    dispman_element=vc_dispmanx_element_add(dispman_update,
                    dispman_display, 0/*layer*/, &dst_rect, 0/*src*/,
                    &src_rect, DISPMANX_PROTECTION_NONE, 0 /*alpha*/,
                    0/*clamp*/, 0/*transform*/);

    nativewindow.element=dispman_element;
    nativewindow.width=width;
    nativewindow.height=height;
    vc_dispmanx_update_submit_sync(dispman_update);

    surface = eglCreateWindowSurface(display,config,&nativewindow,NULL);

    eglMakeCurrent(display,surface,surface,context);
#else
    width=inho->current_w;
    height=inho->current_h;
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS,0);
    SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL,1);
    SDL_SetVideoMode(width,height,32,SDL_FULLSCREEN+SDL_OPENGL);
#endif
    SDL_ShowCursor(0);

    glClearColor(0,0,0,1);
    glClear(GL_COLOR_BUFFER_BIT+GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);

    // Scale window and coords according to the buffer and resolution
    hx=width/2,
    hy=height/2;

    // Video output! Need borders and pixels are not square
    if((width==720 && height==576) || (width==720 && height==480))
    {
        int xasp=59,yasp=54;

        if(height==480) // Never Twice Same Color
        {
            xasp=10;
            yasp=11;
        }

        if((float)xres/(float)yres < 4.0/3.0)
            hx=height*yasp*xres/xasp/yres/2;
        else
            hy=width*xasp*yres/yasp/xres/2;

        // Borders
        hx=hx*88/100;
        hy=hy*88/100;
    }
    else // Square pix
    {
        // Wider or higher than native
        if(width!=xres || height!=yres)
        {
            if((float)xres/(float)yres < (float)width/(float)height)
                hx=height*xres/yres/2;
            else
                hy=width*yres/xres/2;
        }
    }

    glViewport(width/2-hx,height/2-hy,hx*2,hy*2);
    glEnable(GL_SCISSOR_TEST);
    glScissor(width/2-hx,height/2-hy,hx*2,hy*2);
}

void koelli_close(void)
{
    glClear(GL_COLOR_BUFFER_BIT);
#ifdef RASPI
    eglSwapBuffers(display,surface);

    eglMakeCurrent(display,EGL_NO_SURFACE,EGL_NO_SURFACE,EGL_NO_CONTEXT);

    eglDestroySurface(display,surface);
    eglDestroyContext(display,context);
    eglTerminate(display);
#endif
    SDL_Quit();
}

void koelli_flip(void)
{
    glFlush();
#ifdef RASPI
    eglSwapBuffers(display,surface);
#else
    SDL_GL_SwapBuffers();
#endif
}

int koelli_exitevent(void)
{
    SDL_Event e;

    while(SDL_PollEvent(&e)>0)
    {
        if(e.key.keysym.sym==SDLK_ESCAPE)
            return 1;
    }
    return 0;
}
