
#include <math.h>
#ifdef RASPI
 #include <GLES/gl.h>
#else
 #include <SDL_opengl.h>
#endif

#include <stdlib.h>

// playerin laskuri
extern int syna_counter;
extern short mixed[];

// virkistystaajuus max / Hz
#define RATE 75.0

// parametrien muutosnopeus 0-1
#define PARAMFACTOR 0.03
#define PARAMMAX 5.0

enum {
    TENSION=1,
    CONTINUITY,
    BIAS,
    COLORF,
    TRANSLATEZ,
    ROTX,
    RZSPD,
    TAUSTA

};
// muuttaa flotarin kiinten pisteen luvuksi
#define F(f) ((f/PARAMMAX)*128)
// muunnos takaisin
#define FL(i) ((float)i*PARAMMAX/128.0)
#define NOF_PARAMS 9

// paramaterit:
// tahti tension continuity bias vrikerroin zpos zrot rzspd tausta
signed char param[]={
    0, F(4.99),  F(3.99),F(0.0),  F(0.0),F(-0.2),F(0.6),F(0.01),F(0.9),
    8, F(1.99),  F(3.99),F(2.0),  F(0.0),F(-1.3),F(1.3),F(0.01),F(0.9),
    12, F(1.99),  F(3.99),F(2.0),  F(0.2),F(-0.3),F(-0.6),F(0.01),F(0.9),
    16,F(0.0),  F(4.99),  F(2.0),  F(1.0),F(-0.7),F(2.5),F(0.5),F(1.0),
    24,F(0.0),  F(-4.4),  F(0.0),  F(0.6),F(-0.4),F(1.5),F(-0.7),F(1.0),
    32,F(1.00),  F(2.00),  F(1.4),  F(1.0),F(-0.5),F(0.5),F(0.0),F(0.9),
    34,F(0.0),  F(4.99),  F(2.0),  F(1.0),F(-0.7),F(0.5),F(1.0),F(1.5),
    36,F(0.0),  F(4.99),  F(2.0),  F(1.0),F(-0.7),F(-1.0),F(1.0),F(1.5),
    40,F(1.00),  F(2.00),  F(1.4),  F(1.0),F(-0.4),F(0.0),F(1.0),F(0.0),
    44,F(-1.00),  F(-2.00),  F(0.0),  F(1.0),F(-0.4),F(0.0),F(1.5),F(0.0),
    46,F(-1.00),  F(-2.00),  F(0.0),  F(1.0),F(-0.3),F(0.0),F(4.5),F(0.0),
    47,F(-1.00),  F(-2.00),  F(0.0),  F(1.0),F(-0.2),F(0.5),F(2.5),F(0.0),
    48,F(0.000),  F(0.000),  F(0.0),  F(1.0),F(-0.7),F(2.5),F(0.2),F(1.0),
    56,F(0.000),  F(0.000),  F(0.0),  F(0.0),F(-2.9),F(-3.5),F(2.2),F(1.0),
    57,F(0.000),  F(0.000),  F(0.0),  F(0.0),F(0.9),F(-3.5),F(2.2),F(1.0),
    58,F(0.000),  F(0.000),  F(0.0),  F(0.0),F(0.9),F(-3.5),F(2.2),F(0.0),
    59,F(0.000),  F(0.000),  F(0.0),  F(0.0),F(4.9),F(-3.5),F(2.2),F(0.0),
};

float paramf[NOF_PARAMS+1]={0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};

int beat;
#define tension (paramf[TENSION])
#define continuity (paramf[CONTINUITY])
#define bias (paramf[BIAS])
#define color_f (paramf[COLORF])

typedef float Vect[3];
Vect kukka[7]={
    {0.0,0.5,0.0},
    {0.0,0.0,0.0},
    {0.1,-0.2,0.0},
    {0.3,-0.3,0.0},
    {0.5,-0.3,0.0},
    {0.6,-0.2,0.0},
    {0.6,-0.5,0.0}};

Vect kukka_morph[7]={
    {0.0,0.5,0.0},
    {0.0,0.0,0.0},
    {0.2,-0.2,0.0},
    {0.3,-0.15,0.0},
    {0.2,0.0,0.0},
    {0.1,0.0,0.0},
    {0.0,0.1,0.0}};

#define BLKSIZE 2048
float buf[BLKSIZE];

#define FACTOR 0.35
float max_sample;
void analyysi(signed short *dest) {
    int i;
    float w,max=0;

    for (i=0; i<BLKSIZE; i++) {
	w=dest[i*2]/32768.0/2;
	buf[i]=w/5.0;
	if (fabs(w)>=max)
	    max=fabs(w);
    }

    max_sample=max*FACTOR+(1.0-FACTOR)*max_sample;
    if (beat>=48)
        max_sample=0.0;
}

void splineTCP(float u, Vect * control, Vect * result)
{
    // u on sijainti kontrollipisteiden 1 ja 2 vlill (0..1)
    // control pointteri kontrollipisteisiin
    // *x, *y, *z pointterit lopputulokselle
    int n;

    for(n=0;n<2;n++)
        (*result)[n] = (2*u*u*u - 3*u*u + 1)*control[1][n]
	 + (-2*u*u*u + 3*u*u)*control[2][n]
	 + (u*u*u - 2*u*u + u)*(0.5*(1-tension)*((1+bias)*(1-continuity)*(control[1][n]-control[0][n])
						+ (1-bias)*(1+continuity)*(control[2][n]-control[1][n])))
	 + (u*u*(u-1))*(0.5*(1-tension)*((1+bias)*(1+continuity)*(control[2][n]-control[1][n])
					  + (1-bias)*(1-continuity)*(control[3][n]-control[2][n])));
}
float timef;

static float taple[3*1000],*ta,norm[3*1000],*no,
             line1[3*1000],*l1;
static int maxi=0;

void spline3DMorph(float factor)
{
#define NBTW 16

    int i,j,k,c,en;
    static Vect r,r_morph,n;
    static float rf[17*NBTW*3];

    c=0;
    for (i=0; i<sizeof(kukka)/sizeof(Vect)-3; i++) {
    	for (j=0; j<NBTW; j++) {
	        splineTCP((float)j/NBTW, &kukka[i], &r);
	        splineTCP((float)j/NBTW, &kukka_morph[i], &r_morph);
	        for (k=0; k<3; k++)
		        rf[c*3+k]=(1.0-factor)*r[k]+factor*r_morph[k];

    	    rf[c*3+2]=sin(c*M_PI/64)*0.07;
	        c++;
    	}
    }

    maxi=c*6;
    ta=taple;
    no=norm;
    l1=line1;

    for (i=0; i<(c-1); i++)
    {
        n[0]=(rf[(i+1)*3+2]-rf[i*3+2]);
	    n[1]=-(rf[(i+1)*3]-rf[i*3]);
	    n[2]=rf[(i+1)*3+1]-rf[i*3+1];

        rf[i*3+1]+=sinf(i*NBTW/16*0.5+timef*0.006)*max_sample*(0.006*(1.0+i*0.6));
        rf[i*3+2]+=sinf(i*NBTW/16*0.05+timef*0.002)*max_sample*(0.001*(1.0+i*1.0));

        for(en=0;en<6;en++)
        {
            *no++=n[0];
            *no++=n[1];
            *no++=n[2];
        }

        *ta++=1.0*rf[i*3]; *ta++=-rf[i*3+2]; *ta++=rf[i*3+1]*1.1;
        *ta++=1.0*rf[i*3+3]; *ta++=-rf[i*3+3+2]; *ta++=rf[i*3+3+1]*1.1;
        *ta++=1.0*rf[i*3+3]; *ta++=+rf[i*3+3+2]; *ta++=rf[i*3+3+1]*1.1;

        *ta++=1.0*rf[i*3]; *ta++=-rf[i*3+2]; *ta++=rf[i*3+1]*1.1;
        *ta++=1.0*rf[i*3+3]; *ta++=+rf[i*3+3+2]; *ta++=rf[i*3+3+1]*1.1;
        *ta++=1.0*rf[i*3]; *ta++=+rf[i*3+2]; *ta++=rf[i*3+1]*1.1;

        *l1++=1.0*rf[i*3+3]; *l1++=+rf[i*3+3+2]; *l1++=rf[i*3+3+1]*1.1;
        *l1++=1.0*rf[i*3]; *l1++=+rf[i*3+2]; *l1++=rf[i*3+1]*1.1;

        *l1++=1.0*rf[i*3+3]; *l1++=-rf[i*3+3+2]; *l1++=rf[i*3+3+1]*1.1;
        *l1++=1.0*rf[i*3]; *l1++=-rf[i*3+2]; *l1++=rf[i*3+1]*1.1;
    }
}

extern char quit;

int osa(int time)
{
    int t;
    float f;
    //static int kelkka=0;
    static GLfloat light_ambient[]={0.2,0.2,0.2,0.0f};
    static float rz=0.0;
    int i,j,n;

    timef=time;
    beat=((float)(syna_counter/44100.0)*(125.0/60.0)-0.3)/4.0;

    if (beat>=59) {
        quit=1;
    }

    analyysi(&mixed[2*time*441/10]);

    for (i=0; i< sizeof(param)/NOF_PARAMS-1 ; i++)
    {
    	if (beat<param[(i+1)*NOF_PARAMS])
        {
	        for (j=1; j<=NOF_PARAMS; j++)
		        paramf[j]=
		                  (1.0-PARAMFACTOR)*paramf[j]+
                          PARAMFACTOR*param[i*NOF_PARAMS+j]*PARAMMAX/128.0;
	        break;
	    }
    }
    f=paramf[TAUSTA];
    glClearColor(f,f,f,1.0);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glEnable(GL_NORMALIZE);
    glEnable(GL_DEPTH_TEST);
    glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST);

    /* InitViewPort */
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity();
#ifdef RASPI
    glFrustumf(-0.01,0.01, -0.0075,0.0075, 0.01,1135.0);
#else
    glFrustum(-0.01,0.01, -0.0075,0.0075, 0.01,1135.0);
#endif

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    glTranslatef(0.0,0.0,(paramf[TRANSLATEZ]));

    glRotatef(-(paramf[ROTX])*90.0/5.0,1.0,0.0,0.0);
    rz+=paramf[RZSPD]*1.50;
    glRotatef(rz,0.0,0.0,1.0);

    static GLfloat linecoord[BLKSIZE/2*3],colors[BLKSIZE/2*4];

    glEnableClientState(GL_VERTEX_ARRAY);
    /* aaltoMuoto */
    if (beat<48)
    {
    	glDisable(GL_LIGHTING);
        glEnableClientState(GL_COLOR_ARRAY);
        glVertexPointer(3,GL_FLOAT,0,linecoord);
        glColorPointer(4,GL_FLOAT,0,colors);
	    glLineWidth(3.0);

        for (i=0; i<BLKSIZE/2; i++)
        {
            f=1.0-sinf(i*M_PI/(BLKSIZE/2))*1.0;
            colors[i*4+0]=colors[i*4+1]=colors[i*4+2]=f;
            colors[i*4+3]=1.0f;
            linecoord[i*3+0]=2.0*((float)i/(BLKSIZE/2)-0.5);
            linecoord[i*3+1]=buf[i];
            linecoord[i*3+2]=-0.3;
        }

        for(n=0;n<2;n++)
        {
            glDrawArrays(GL_LINE_STRIP,0,BLKSIZE/2);
            glRotatef(90,0,0,1);
        }
    }

  	glEnable(GL_LIGHTING);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glEnable(GL_LIGHT0);
    glEnable(GL_COLOR_MATERIAL);

    glDisableClientState(GL_COLOR_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glVertexPointer(3,GL_FLOAT,0,taple);
    glNormalPointer(GL_FLOAT,0,norm);

    glRotatef(360.0/24.0,0.0,0.0,1.0);
  	spline3DMorph(sin(time*0.001)*0.5+0.5);

    glColor4f(color_f*0.82,color_f*0.87,color_f*0.95,1.0f);
    for (t=0; t<12; t++)
    {
        glDrawArrays(GL_TRIANGLES,0,maxi);
	    glRotatef(360.0/12.0,0.0,0.0,1.0);
    }

    glDisable(GL_LIGHT0);
  	glDisable(GL_LIGHTING);
    glDisableClientState(GL_NORMAL_ARRAY);
    glVertexPointer(3,GL_FLOAT,0,line1);

    glColor4f(0,0,0,1.0f);
    for (t=0; t<12; t++)
    {
        glDrawArrays(GL_LINES,0,maxi*2/3);
	    glRotatef(360.0/12.0,0.0,0.0,1.0);
    }

    return 1;
}
